/*
  +----------------------------------------------------------------------+
  | PHP Version 5                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2004 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3.0 of the PHP license,       |
  | that is bundled with this package in the file LICENSE, and is        |
  | available at through the world-wide-web at                           |
  | http://www.php.net/license/3_0.txt.                                  |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Author: Pierre-Alain Joye <paj@pearfr.org>                           |
  +----------------------------------------------------------------------+

  $Id: date.c,v 1.14 2004/03/05 12:21:10 pajoye Exp $
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
#include "php_date.h"
#include "t_date.h"

typedef date_int    (*date_prop_get)(Date*);
typedef DateBoolean (*date_prop_set)(Date*, date_int);

typedef struct _date_prop_inf_struct {
	char* name;
	size_t size;
	date_prop_get get;
	date_prop_set set;
	ulong  h;
} date_prop_inf_struct;

date_prop_inf_struct date_prop_inf[4] = {
	{"day",         sizeof("day"),         (date_prop_get)date_get_day,    (date_prop_set)date_set_day},
	{"month",       sizeof("month"),       (date_prop_get)date_get_month,  (date_prop_set)date_set_month},
	{"year",        sizeof("year"),        (date_prop_get)date_get_year,   (date_prop_set)date_set_year},
	{"julian_days", sizeof("julian_days"), (date_prop_get)date_get_julian, (date_prop_set)date_set_julian}
};

#define DATE_ASSERT_RETURN_BOOL(expr) \
	if (expr) { \
		RETURN_TRUE; \
	} else {\
		RETURN_FALSE; \
	}

#define DATE_ASSERT_RETURN_OBJECT(expr, obj) \
	if (expr) { \
		RETURN_ZVAL(obj, 1, 0); \
	} else {\
		RETURN_FALSE; \
	}

#define DATE_ASSERT_RETURN_WARNING(expr, warnmsg) \
	if (expr) { \
		RETURN_TRUE; \
	} else {\
		php_error_docref(NULL TSRMLS_CC, E_WARNING, warnmsg); \
		RETURN_FALSE; \
	}

#define DATE_ASSERT_RETURN_WARNING_OBJECT(expr, obj, warnmsg) \
	if (expr) { \
		RETURN_ZVAL(obj, 1, 0); \
	} else {\
		php_error_docref(NULL TSRMLS_CC, E_WARNING, warnmsg); \
		RETURN_FALSE; \
	}


#define DATE_REGISTER_OBJECT(_type, _object, _ptr) \
	{ \
		date_object *obj; \
		obj = (date_object*)zend_object_store_get_object(_object TSRMLS_CC); \
		obj->ptr = _ptr; \
	}

#define DATE_FROM_OBJECT(date, object) \
	{ \
		date_object *obj = (date_object*) zend_object_store_get_object(object TSRMLS_CC); \
		pdate = obj->ptr; \
		if (!pdate) { \
			php_error_docref(NULL TSRMLS_CC, E_WARNING, "The date wasn't initialized"); \
			RETURN_FALSE; \
		} \
	}

#define DATE_FROM_OBJECT_EX(obj, date, object, retval) \
	{ \
		obj = (date_object*) zend_object_store_get_object(object TSRMLS_CC); \
		pdate = obj->ptr; \
		if (!pdate) { \
			php_error_docref(NULL TSRMLS_CC, E_WARNING, "The date wasn't initialized"); \
			return retval; \
		} \
	}

#define DATE_FROM_ZVAL(_pdate, zvalue) \
	{ \
		if (Z_TYPE_P(zvalue) == IS_OBJECT) { \
			date_object *obj = (date_object*) zend_object_store_get_object(zvalue TSRMLS_CC); \
			_pdate = obj->ptr; \
			if (!_pdate) { \
				php_error_docref(NULL TSRMLS_CC, E_WARNING, "The date wasn't initialized"); \
				RETURN_FALSE; \
			} \
		} else if (Z_TYPE_P(zvalue) == IS_RESOURCE)  { \
			ZEND_FETCH_RESOURCE(_pdate, Date *, &zvalue, -1, "date_time", le_date); \
		} else { \
			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid date variable."); \
			RETURN_FALSE; \
		} \
	}

#define DATE_REGISTER_RESOURCE(res) \
	if (res) { \
		ZEND_REGISTER_RESOURCE(return_value, res, le_date); \
	} else { \
		RETURN_FALSE; \
	}

/* True global resources - no need for thread safety here */
static int le_date;

/* If you declare any globals in php_date.h uncomment this:*/
/*ZEND_DECLARE_MODULE_GLOBALS(date)*/


static zend_object_handlers date_object_handlers;

/* {{{ date_fetch_object()
 */

static inline date_object *date_fetch_object(zval *object TSRMLS_DC)
{
	return (date_object *) zend_object_store_get_object(object TSRMLS_CC);
}
/* }}} */

/*
 * FIXME: Using debug mode and date shared, that causes a crash on exit if
 * the date object is destroyed by ZE and not in userland ($d=null).
 * See test_010.phpt.
 * <?php $d = new Date(); $d=null ?> no crash
 * <?php $d = new Date(); ?> crash
 * <?php $d = new Date();$d=null ?> crash if zend_objects_destroy_object called
 */
/* {{{ date_objects_dtor
 */
static void date_objects_dtor(void *object, zend_object_handle handle TSRMLS_DC)
{
	date_object *date = (date_object *) object;

	if (date->zo.properties) {
		zend_hash_destroy(date->zo.properties);
		FREE_HASHTABLE(date->zo.properties);
	}

	date_free(date->ptr);

	efree(date);
}
/* }}} */

/* {{{ date_objects_clone
 * Do not expect it to work now :-)
 */
static void date_objects_clone(void *object, void **object_clone TSRMLS_DC)
{
	date_object *intern = (date_object *) object;
	date_object **intern_clone = (date_object **) object_clone;
	
	*intern_clone = emalloc(sizeof(date_object));
	(*intern_clone)->zo.ce = intern->zo.ce;
	(*intern_clone)->zo.in_get = 0;
	(*intern_clone)->zo.in_set = 0;
	
	ALLOC_HASHTABLE((*intern_clone)->zo.properties);
	zend_hash_init((*intern_clone)->zo.properties, 0, NULL, ZVAL_PTR_DTOR, 0);
	zend_hash_copy((*intern_clone)->zo.properties, intern->zo.properties, (copy_ctor_func_t) zval_add_ref, NULL, sizeof(zval *));
	
	/* alloc memory for your custom data structure
	 * and assign it here
	 * intern->ptr = emalloc(sizeof(underlying_data));
	 */
}
/* }}} */

/* {{{ date_objects_new
 */
PHP_DATE_API zend_object_value date_objects_new(zend_class_entry *class_type TSRMLS_DC)
{
	zend_object_value retval;
	date_object *intern;

	intern = emalloc(sizeof(date_object));
	intern->zo.ce = class_type;
	intern->zo.in_get = 0;
	intern->zo.in_set = 0;
	intern->zo.properties = NULL;

	ALLOC_HASHTABLE(intern->zo.properties);
	zend_hash_init(intern->zo.properties, 0, NULL, ZVAL_PTR_DTOR, 0);
	zend_hash_copy(intern->zo.properties, &class_type->default_properties, (copy_ctor_func_t) zval_add_ref, NULL, sizeof(zval *));

	/* retval.handle = zend_objects_store_put(intern, date_objects_dtor, date_objects_clone TSRMLS_CC); */
	retval.handle = zend_objects_store_put(intern, date_objects_dtor, NULL, NULL TSRMLS_CC);
	retval.handlers = (zend_object_handlers *) &date_object_handlers;

	return retval;
}
/* }}} */

zval *date_read_property(zval *object, zval *member, zend_bool silent TSRMLS_DC)
{
	char * prop;
	int i;
	long t;

	convert_to_string_ex(&member);
	prop = Z_STRVAL_P(member);

	/*
	 * Keeping many DATE_FROM_OBJECT avoids
	 * useless object fetching if the prop does
	 * exist or is not set ;)
	 */
	for (i=0; i<sizeof(date_prop_inf) / sizeof(date_prop_inf[0]); i++) {
		if (!strncmp(prop, date_prop_inf[i].name, date_prop_inf[i].size - 1)) {
			Date * pdate;
			zval ** value; 
			date_object *obj;
			zval *v;
			DATE_FROM_OBJECT_EX(obj, pdate, object, EG(uninitialized_zval_ptr));
			zend_hash_quick_find(obj->zo.properties, date_prop_inf[i].name, date_prop_inf[i].size, date_prop_inf[i].h, (void**)&value);
			ZVAL_LONG(*value, date_prop_inf[i].get(pdate));
/*		(*value)->refcount++;*/
			return *value;
		}
	}
	return std_object_handlers.read_property(object, member, silent TSRMLS_CC);
}

void date_write_property(zval *object, zval *member, zval *value TSRMLS_DC)
{
	char * prop;
	Date * pdate;
	int i;

	convert_to_string_ex(&member);
	prop = Z_STRVAL_P(member);

	/*
	 * Keeping many DATE_FROM_OBJECT avoids
	 * useless object fetching if the prop does
	 * exist or is not set ;)
	 */
	for (i=0; i<sizeof(date_prop_inf) / sizeof(date_prop_inf[0]); i++) {
		if (!strncmp(prop, date_prop_inf[i].name, date_prop_inf[i].size - 1)) {
			date_object *obj;

			convert_to_long_ex(&value);
			DATE_FROM_OBJECT_EX(obj, pdate, object, /**/);
			date_prop_inf[i].set(pdate, (date_int)Z_LVAL_P(value));
			return;
		}
	}
	std_object_handlers.write_property(object, member, value TSRMLS_CC);
}

static HashTable *date_properties_get (zval *object TSRMLS_DC)
{
	date_object *obj = date_fetch_object(object TSRMLS_CC);
	HashTable *ht = obj->zo.properties;
	Date * pdate = obj->ptr;
	zval ** value; 
	int i;

	if (pdate) {
		for (i=0; i<sizeof(date_prop_inf) / sizeof(date_prop_inf[0]); i++) {
			zend_hash_quick_find(obj->zo.properties, date_prop_inf[i].name, date_prop_inf[i].size, date_prop_inf[i].h, (void**)&value);
			ZVAL_LONG(*value, date_prop_inf[i].get(pdate));
		}
	} else {
		for (i=0; i<sizeof(date_prop_inf) / sizeof(date_prop_inf[0]); i++) {
			zend_hash_quick_find(obj->zo.properties, date_prop_inf[i].name, date_prop_inf[i].size, date_prop_inf[i].h, (void**)&value);
			ZVAL_NULL(*value);
		}
	}

	return ht;
}


/* {{{ date_functions[]
 *
 * Every user visible function must have an entry in date_functions[].
 */
function_entry date_functions[] = {
	PHP_FE(date_new,	NULL)
	PHP_FE(date_free,	NULL)
	PHP_FE(date_new_julian,	NULL)
	PHP_FE(date_new_dmy,	NULL)
	PHP_FE(date_new_now,	NULL)
	PHP_FE(date_valid,	NULL)
	PHP_FE(date_valid_julian,	NULL)
	PHP_FE(date_valid_dmy,	NULL)
	PHP_FE(date_set_dmy,	NULL)
	PHP_FE(date_set_day,	NULL)
	PHP_FE(date_get_day,	NULL)
	PHP_FE(date_set_month,	NULL)
	PHP_FE(date_get_month,	NULL)
	PHP_FE(date_set_year,	NULL)
	PHP_FE(date_get_year,	NULL)
	PHP_FE(date_set_julian,	NULL)
	PHP_FE(date_get_julian,	NULL)
	PHP_FE(date_is_leap_year,	NULL)
	PHP_FE(date_add_days,	NULL)
	PHP_FE(date_sub_days,	NULL)
	PHP_FE(date_add_weeks,	NULL)
	PHP_FE(date_sub_weeks,	NULL)
	PHP_FE(date_add_months,	NULL)
	PHP_FE(date_sub_months,	NULL)
	PHP_FE(date_add_years,	NULL)
	PHP_FE(date_sub_years,	NULL)
	PHP_FE(date_days_between,	NULL)
	PHP_FE(date_compare,	NULL)
	PHP_FE(date_order,	NULL)
	PHP_FE(date_clamp,	NULL)
	PHP_FE(date_get_weekday,	NULL)
	PHP_FE(date_get_day_of_year,	NULL)
	PHP_FE(date_get_week_of_year,	NULL)
	PHP_FE(date_get_isoweek_of_year,	NULL)
	PHP_FE(date_set_to_weekday,	NULL)
	PHP_FE(date_get_to_weekday,	NULL)
	PHP_FE(date_set_to_last_month_day,	NULL)
	PHP_FE(date_get_to_last_month_day,	NULL)
	PHP_FE(date_set_to_prev_week_day,	NULL)
	PHP_FE(date_get_to_prev_week_day,	NULL)
	PHP_FE(date_set_to_next_weekday,	NULL)
	PHP_FE(date_get_to_next_weekday,	NULL)
	PHP_FE(date_set_to_weekday_in_same_week,	NULL)
	PHP_FE(date_get_to_weekday_in_same_week,	NULL)
	PHP_FE(date_set_to_week,	NULL)
	PHP_FE(date_get_to_week,	NULL)
	PHP_FE(date_set_last_dow,	NULL)
	PHP_FE(date_get_last_dow,	NULL)
	PHP_FE(date_set_first_dow,	NULL)
	PHP_FE(date_get_first_dow,	NULL)
	PHP_FE(date_get_days_in_month,	NULL)
	PHP_FE(date_get_weeks_of_year,	NULL)
	PHP_FE(date_strftime,		NULL)
	PHP_FE(date_get_array,		NULL)
	PHP_FE(date_get_easter,	NULL)
		{NULL, NULL, NULL}	/* Must be the last line in date_functions[] */
};
/* }}} */

/* {{{ date_module_entry
 */
zend_module_entry date_module_entry = {
#if ZEND_MODULE_API_NO >= 20010901
	STANDARD_MODULE_HEADER,
#endif
	"date_time",
	date_functions,
	PHP_MINIT(date),
	PHP_MSHUTDOWN(date),
	NULL,//	PHP_RINIT(date),		/* Replace with NULL if there's nothing to do at request start */
	NULL,//PHP_RSHUTDOWN(date),	/* Replace with NULL if there's nothing to do at request end */
	PHP_MINFO(date),
#if ZEND_MODULE_API_NO >= 20010901
	"0.1", /* Replace with version number for your extension */
#endif
	STANDARD_MODULE_PROPERTIES
};
/* }}} */

#ifdef COMPILE_DL_DATE
ZEND_GET_MODULE(date)
#endif

static void php_date_free(zend_rsrc_list_entry *rsrc TSRMLS_DC)
{
	if (rsrc->ptr) {
		Date *d = (Date *)rsrc->ptr;
		date_free(d);
		rsrc->ptr = NULL;
	}
}

/* {{{ PHP_FUNCTION(date_ctor)
 */
PHP_FUNCTION(date_ctor)
{
	zval *this = getThis();
	long d = 0, m = 0, y = 0, weekstart = DATE_STARTWEEK_MONDAY;
	Date *date;

	switch(ZEND_NUM_ARGS()) {
		case 0:
		case 1:
			if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|l", &d)) {
				RETURN_FALSE;
			}
			date = date_new(weekstart);
			break;
		case 2:
			if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ll", &d, &weekstart)) {
				RETURN_FALSE;
			}
			date = date_new_julian(d, weekstart TSRMLS_CC);
			break;
		case 3:
		case 4:
			if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "lll|l", &d, &m, &y, &weekstart)) {
				RETURN_FALSE;
			}
			date = date_new_dmy(d, m, y, weekstart);
			break;
		default:
			WRONG_PARAM_COUNT;
	}
	DATE_REGISTER_OBJECT(date, this, date)
}

/* }}} */

/* {{{ date_class_functions
 */
function_entry date_class_functions[] = {
	PHP_ME_MAPPING(date, date_ctor, NULL)     /* constructor */
	PHP_ME_MAPPING(setDMY, date_set_dmy, NULL)
	PHP_ME_MAPPING(isValid, date_valid, NULL)
	PHP_ME_MAPPING(setDay, date_set_day, NULL)
	PHP_ME_MAPPING(getDay, date_get_day, NULL)
	PHP_ME_MAPPING(setMonth, date_set_month, NULL)
	PHP_ME_MAPPING(getMonth, date_get_month, NULL)
	PHP_ME_MAPPING(setYear, date_set_year, NULL)
	PHP_ME_MAPPING(getYear, date_get_year, NULL)
	PHP_ME_MAPPING(setJulian, date_set_julian, NULL)
	PHP_ME_MAPPING(getJuliaan, date_get_julian, NULL)
	PHP_ME_MAPPING(isLeap, date_is_leap_year, NULL)
	PHP_ME_MAPPING(addDays, date_add_days, NULL)
	PHP_ME_MAPPING(subDays, date_sub_days, NULL)
	PHP_ME_MAPPING(addWeeks, date_add_weeks, NULL)
	PHP_ME_MAPPING(subMonths, date_sub_months, NULL)
	PHP_ME_MAPPING(addMonths, date_add_months, NULL)
	PHP_ME_MAPPING(subWeeks, date_sub_weeks, NULL)
	PHP_ME_MAPPING(addYears, date_add_years, NULL)
	PHP_ME_MAPPING(subYears, date_sub_years, NULL)
	PHP_ME_MAPPING(getArray, date_get_array, NULL)
	PHP_ME_MAPPING(getWeekday, date_get_weekday, NULL)
	PHP_ME_MAPPING(getDayOfYear, date_get_day_of_year, NULL)
	PHP_ME_MAPPING(getWeekOfYear, date_get_week_of_year, NULL)
	PHP_ME_MAPPING(getISOWeekOfYear, date_get_isoweek_of_year, NULL)
	PHP_ME_MAPPING(setToWeekday, date_set_to_weekday, NULL)
	PHP_ME_MAPPING(setToLastMonthDay, date_set_to_last_month_day, NULL)
	PHP_ME_MAPPING(setFirstDow, date_set_first_dow, NULL)
	PHP_ME_MAPPING(setLastDow, date_set_last_dow, NULL)
	PHP_ME_MAPPING(setToPreviousWeekday, date_set_to_prev_week_day, NULL)
	PHP_ME_MAPPING(setToNextWeekday, date_set_to_next_weekday, NULL)
	PHP_ME_MAPPING(setToWeekdayInSameWeek, date_set_to_weekday_in_same_week, NULL)
	PHP_ME_MAPPING(setToWeek, date_set_to_week, NULL)
	{NULL, NULL, NULL}
};
/* }}} */

/* {{{ PHP_MINIT_FUNCTION
 */
PHP_MINIT_FUNCTION(date)
{
	int i;
	zend_class_entry _date_class_entry;

	INIT_CLASS_ENTRY(_date_class_entry, "date", date_class_functions);
	_date_class_entry.create_object = date_objects_new;
	date_class_entry = zend_register_internal_class(&_date_class_entry TSRMLS_CC);

	for (i=0; i < sizeof(date_prop_inf) / sizeof(date_prop_inf[0]); i++) {
		date_prop_inf[i].h = zend_hash_func(date_prop_inf[i].name, date_prop_inf[i].size);
		zend_declare_property_null(date_class_entry , date_prop_inf[i].name, date_prop_inf[i].size-1, ZEND_ACC_PUBLIC TSRMLS_CC);
	}

	/* copy the standard object handlers to you handler table */
	memcpy(&date_object_handlers, zend_get_std_object_handlers(), sizeof(zend_object_handlers));

/*	date_object_handlers.clone_obj = zend_objects_store_clone_obj;*/
	date_object_handlers.read_property = date_read_property;
	date_object_handlers.write_property = date_write_property;
	date_object_handlers.get_properties = date_properties_get;
    date_object_handlers.get_property_ptr_ptr =  NULL;

	le_date = zend_register_list_destructors_ex(php_date_free, NULL, "date_time", module_number);
	REGISTER_LONG_CONSTANT("DATE_BAD_WEEKDAY",	DATE_BAD_WEEKDAY, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("DATE_MONDAY",	DATE_MONDAY, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("DATE_TUESDAY",	DATE_TUESDAY, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("DATE_WEDNESDAY",DATE_WEDNESDAY, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("DATE_THURSDAY", DATE_THURSDAY, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("DATE_FRIDAY",	DATE_FRIDAY, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("DATE_SATURDAY", DATE_SATURDAY, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("DATE_SUNDAY",	DATE_SUNDAY, CONST_CS | CONST_PERSISTENT);

	REGISTER_LONG_CONSTANT("DATE_BAD_MONTH", DATE_BAD_MONTH, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("DATE_JANUARY", DATE_JANUARY, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("DATE_FEBRUARY",	DATE_FEBRUARY, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("DATE_MARCH", DATE_MARCH, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("DATE_APRIL", DATE_APRIL, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("DATE_MAY", DATE_MAY, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("DATE_JUNE", DATE_JUNE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("DATE_JULY", DATE_JULY, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("DATE_AUGUST", DATE_AUGUST, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("DATE_SEPTEMBER", DATE_SEPTEMBER, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("DATE_OCTOBER", DATE_OCTOBER, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("DATE_NOVEMBER", DATE_NOVEMBER, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("DATE_DECEMBER", DATE_DECEMBER, CONST_CS | CONST_PERSISTENT);

	REGISTER_LONG_CONSTANT("DATE_STARTWEEK_SUNDAY", DATE_STARTWEEK_SUNDAY, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("DATE_STARTWEEK_MONDAY", DATE_STARTWEEK_MONDAY, CONST_CS | CONST_PERSISTENT);

	REGISTER_LONG_CONSTANT("DATE_JULIAN_MIN", DATE_JULIAN_MIN, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("DATE_JULIAN_MAX", DATE_JULIAN_MAX, CONST_CS | CONST_PERSISTENT);

	REGISTER_LONG_CONSTANT("DATE_MONTH_MIN", DATE_MONTH_MIN, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("DATE_MONTH_MAX", DATE_MONTH_MAX, CONST_CS | CONST_PERSISTENT);

	REGISTER_LONG_CONSTANT("DATE_BAD_JULIAN", DATE_BAD_JULIAN, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("DATE_BAD_DAY", DATE_BAD_DAY, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("DATE_BAD_YEAR", DATE_BAD_YEAR, CONST_CS | CONST_PERSISTENT);

	return SUCCESS;
}
/* }}} */

/* {{{ PHP_MSHUTDOWN_FUNCTION
 */
PHP_MSHUTDOWN_FUNCTION(date)
{
	return SUCCESS;
}
/* }}} */

/* {{{ PHP_MINFO_FUNCTION
 */
PHP_MINFO_FUNCTION(date)
{
	php_info_print_table_start();
	php_info_print_table_header(2, "date support", "enabled");
	php_info_print_table_end();
}
/* }}} */

/* {{{ proto resource date_new([int weekstart])
   create a new date, weekstart defines if the week begins on monday or sunday DATE_MONDAY or DATE_SUNDAY */
PHP_FUNCTION(date_new)
{
	long weekstart = DATE_STARTWEEK_MONDAY;
	Date *date;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|l", &weekstart) == FAILURE) {
		RETURN_FALSE;
	}

	date = date_new(weekstart);
	DATE_REGISTER_RESOURCE(date);
}
/* }}} */

/* {{{ proto boolean date_free(resource date)
   free a date */
PHP_FUNCTION(date_free)
{
	zval *date;
	Date *pdate;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &date) == FAILURE) {
		RETURN_FALSE;
	}

	ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);

	zend_list_delete(Z_RESVAL_P(date));

	RETURN_TRUE;
}
/* }}} */

/* {{{ proto resource date_new_julian(int julian_days[, int weekstart])
   create a date using julian_days, or the current date */
PHP_FUNCTION(date_new_julian)
{
	long julian_days = 0, weekstart = DATE_STARTWEEK_MONDAY;
	Date *date;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l|l", &julian_days, &weekstart) == FAILURE) {
		RETURN_FALSE;
	}

	date = date_new_julian(julian_days, weekstart TSRMLS_CC);
	DATE_REGISTER_RESOURCE(date);
}
/* }}} */

/* {{{ proto resource date_new_dmy(int d , int m , int y, [int weekstart])
   create a date using day d, month m and year y, or the current date */
PHP_FUNCTION(date_new_dmy)
{
	long d, m, y, weekstart = DATE_STARTWEEK_MONDAY;
	Date *date;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "lll|l", &d, &m, &y, &weekstart) == FAILURE) {
		RETURN_FALSE;
	}

	date = date_new_dmy(d, m, y, weekstart);
	DATE_REGISTER_RESOURCE(date);
}
/* }}} */


/* {{{ proto resource date_new_now([int weekstart])
   create a date using day d, month m and year y, or the current date */
PHP_FUNCTION(date_new_now)
{
	long weekstart = DATE_STARTWEEK_MONDAY;
	Date *date;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|l", &weekstart) == FAILURE) {
		RETURN_FALSE;
	}

	date = date_new_now(weekstart TSRMLS_CC);
	DATE_REGISTER_RESOURCE(date);
}
/* }}} */

/* {{{ proto bool date_valid(resource date)
   returns true if the date 'date' is valid */
PHP_FUNCTION(date_valid)
{
	zval *date;
	Date *pdate;
	zval *object = getThis();

	if (object) {
		DATE_FROM_OBJECT(pdate, object);
	} else {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &date) == FAILURE) {
			RETURN_FALSE;
		}
		ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);
	}
	DATE_ASSERT_RETURN_BOOL(date_valid(pdate));
}
/* }}} */

/* {{{ proto bool date_valid_julian(int j)
   returns true is the julian day j is valid */
PHP_FUNCTION(date_valid_julian)
{
	long j;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &j) == FAILURE) {
		RETURN_FALSE;
	}

	DATE_ASSERT_RETURN_BOOL(date_valid_julian(j));
}
/* }}} */

/* {{{ proto bool date_valid_dmy(int d, int m, int y)
   returns true if the date defined by day d, month m and year y is valid */
PHP_FUNCTION(date_valid_dmy)
{
	long d, m, y;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "lll", &d, &m, &y) == FAILURE) {
		RETURN_FALSE;
	}

	DATE_ASSERT_RETURN_BOOL(date_valid_dmy(d, m, y));
}
/* }}} */

/* {{{ proto bool date_set_dmy(int date, int day, int month, int year)
   set the day,month and year for the date 'date', returns true on success */
PHP_FUNCTION(date_set_dmy)
{
	zval *object = getThis();
	long day,month,year;
	zval *date;
	Date *pdate;

	if (object) {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "lll", &day, &month, &year) == FAILURE) {
			RETURN_FALSE;
		}
		DATE_FROM_OBJECT(pdate, object);
		DATE_ASSERT_RETURN_WARNING_OBJECT(date_set_dmy(pdate, day, month, year), object, "Cannot set the day");
	} else if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rlll", &date, &day, &month, &year) == SUCCESS) {
		ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);
		DATE_ASSERT_RETURN_WARNING(date_set_dmy(pdate, day, month, year), "Cannot set the day");
	}
	RETURN_FALSE;
}
/* }}} */

/* {{{ proto bool date_set_day(resource date, int day)
   set the day of month for date 'date' using 'day' */
PHP_FUNCTION(date_set_day)
{
	long day;
	zval *date;
	Date *pdate;
	zval *object = getThis();

	if (object) {
		DATE_FROM_OBJECT(pdate, object);
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &day) == FAILURE) {
			RETURN_FALSE;
		}
	} else {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &date, &day) == FAILURE) {
			RETURN_FALSE;
		}
		ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);
	}
	DATE_ASSERT_RETURN_BOOL(date_set_day(pdate, day));
}
/* }}} */

/* {{{ proto int date_get_day(resource date)
   returns the day of month of the date 'date' */
PHP_FUNCTION(date_get_day)
{
	zval *date;
	Date *pdate;
	zval *object = getThis();

	if (object) {
		DATE_FROM_OBJECT(pdate, object);
	} else {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &date) == FAILURE) {
			RETURN_FALSE;
		}
		ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);
	}
	RETURN_LONG((long)date_get_day(pdate));
}
/* }}} */

/* {{{ proto bool date_set_month(resource date, int month)
   set the month of the date 'date' using 'month' */
PHP_FUNCTION(date_set_month)
{
	long month;
	zval *date;
	Date *pdate;
	zval *object = getThis();

	if (object) {
		DATE_FROM_OBJECT(pdate, object);
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &month) == FAILURE) {
			RETURN_FALSE;
		}
		DATE_ASSERT_RETURN_OBJECT(date_set_month(pdate, month), object);
	} else {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &date, &month) == FAILURE) {
			RETURN_FALSE;
		}
		ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);
		DATE_ASSERT_RETURN_BOOL(date_set_month(pdate, month));
	}
}
/* }}} */

/* {{{ proto int date_get_month(resource date)
   returns the month number of the date 'date' */
PHP_FUNCTION(date_get_month)
{
	zval *date;
	Date *pdate;
	zval *object = getThis();

	if (object) {
		DATE_FROM_OBJECT(pdate, object);
	} else {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &date) == FAILURE) {
			RETURN_FALSE;
		}
		ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);
	}
	RETURN_LONG((long)date_get_month(pdate));
}
/* }}} */

/* {{{ proto bool date_set_year(resource date, int year)
    */
PHP_FUNCTION(date_set_year)
{
	long year;
	zval *date;
	Date *pdate;
	zval *object = getThis();

	if (object) {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &year) == FAILURE) {
			RETURN_FALSE;
		}
		DATE_FROM_OBJECT(pdate, object);
		DATE_ASSERT_RETURN_WARNING_OBJECT(date_set_year(pdate, year), object,"Cannot set the year");
	} else {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &date, &year) == FAILURE) {
			RETURN_FALSE;
		}

		ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);
		DATE_ASSERT_RETURN_WARNING(date_set_year(pdate, year), "Cannot set the year");
	}
}
/* }}} */

/* {{{ proto int date_get_year(resource date)
    */
PHP_FUNCTION(date_get_year)
{
	zval *date;
	Date *pdate;
	zval *object = getThis();

	if (object) {
		DATE_FROM_OBJECT(pdate, object);
	} else {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &date) == FAILURE) {
			RETURN_FALSE;
		}

		ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);
	}
	RETURN_LONG((long)date_get_year(pdate));
}
/* }}} */

/* {{{ proto bool date_set_julian(resource date, int julianday)
   set the julian day of the date 'date' */
PHP_FUNCTION(date_set_julian)
{
	long julianday;
	zval *date;
	Date *pdate;
	zval *object = getThis();

	if (object) {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &date, &julianday) == FAILURE) {
			RETURN_FALSE;
		}
		DATE_FROM_OBJECT(pdate, object);
		DATE_ASSERT_RETURN_WARNING_OBJECT(date_set_julian(pdate, julianday), object, "Cannot set the julian day");
	} else {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &date, &julianday) == FAILURE) {
			RETURN_FALSE;
		}

		ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);
		DATE_ASSERT_RETURN_WARNING(date_set_julian(pdate, julianday), "Cannot set the julian day");
	}
}
/* }}} */

/* {{{ proto int date_get_julian(resource date)
   get the julian date of date 'date' */
PHP_FUNCTION(date_get_julian)
{
	zval *date;
	Date *pdate;
	zval *object = getThis();

	if (object) {
		DATE_FROM_OBJECT(pdate, object);
	} else {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &date) == FAILURE) {
			RETURN_FALSE;
		}

		ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);
	}
	RETURN_LONG((long)date_get_julian(pdate));
}
/* }}} */

/* {{{ proto bool date_is_leap_year(int year)
   returns true if the year 'year' is a leap yeap */
PHP_FUNCTION(date_is_leap_year)
{
	long year;
	zval *object = getThis();
	Date *pdate;

	if (object) {
		DATE_FROM_OBJECT(pdate, object);
		year = date_get_year(pdate);
	} else {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &year) == FAILURE) {
			RETURN_FALSE;
		}
	}
	DATE_ASSERT_RETURN_BOOL(date_is_leap_year((DateYear)year));
}
/* }}} */

/* {{{ proto bool date_add_days(resource date, int days)
   adds 'days' to the date 'date' */
PHP_FUNCTION(date_add_days)
{
	long days;
	zval *date;
	Date *pdate;
	zval *object = getThis();

	if (object) {
		DATE_FROM_OBJECT(pdate, object);
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &days) == FAILURE) {
			RETURN_FALSE;
		}
		DATE_ASSERT_RETURN_OBJECT(date_add_days(pdate, (DateDay) days), object);
	} else {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &date, &days) == FAILURE) {
			RETURN_FALSE;
		}

		ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);
		DATE_ASSERT_RETURN_BOOL(date_add_days(pdate, (DateDay) days));
	}
}
/* }}} */

/* {{{ proto bool date_sub_days(resource date, int days)
   substarts 'days' to the date 'date' */
PHP_FUNCTION(date_sub_days)
{
	long days;
	zval *date;
	Date *pdate;
	zval *object = getThis();

	if (object) {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &days) == FAILURE) {
			RETURN_FALSE;
		}
		DATE_FROM_OBJECT(pdate, object);
		DATE_ASSERT_RETURN_OBJECT(date_sub_days(pdate, (DateDay)days), object);
	} else {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &date, &days) == FAILURE) {
			RETURN_FALSE;
		}

		ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);
		DATE_ASSERT_RETURN_BOOL(date_sub_days(pdate, (DateDay)days));
	}
}
/* }}} */

/* {{{ proto bool date_add_weeks(resource date, int weeks)
   add weeks to 'date' */
PHP_FUNCTION(date_add_weeks)
{
	long weeks;
	zval *date;
	Date *pdate;
	zval *object = getThis();

	if (object) {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &weeks) == FAILURE) {
			RETURN_FALSE;
		}
		DATE_FROM_OBJECT(pdate, object);
		DATE_ASSERT_RETURN_OBJECT( date_add_weeks(pdate, (DateWeek) weeks), object);
	} else {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &date, &weeks) == FAILURE) {
			RETURN_FALSE;
		}

		ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);
		DATE_ASSERT_RETURN_BOOL( date_add_weeks(pdate, (DateWeek) weeks) );
	}
}
/* }}} */

/* {{{ proto bool date_sub_weeks(resource date, int weeks)
   sub weeks to 'date' */
PHP_FUNCTION(date_sub_weeks)
{
	long weeks;
	zval *date;
	Date *pdate;
	zval *object = getThis();

	if (object) {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &weeks) == FAILURE) {
			RETURN_FALSE;
		}
		DATE_FROM_OBJECT(pdate, object);
		DATE_ASSERT_RETURN_OBJECT( date_sub_weeks(pdate, (DateWeek) weeks), object);
	} else {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &date, &weeks) == FAILURE) {
			RETURN_FALSE;
		}

		ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);
		DATE_ASSERT_RETURN_BOOL( date_sub_weeks(pdate, (DateWeek) weeks) );
	}
}
/* }}} */

/* {{{ proto bool date_add_months(resource date, int months)
   add months to 'date' */
PHP_FUNCTION(date_add_months)
{
	long months;
	zval *date;
	Date *pdate;
	zval *object = getThis();

	if (object) {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &months) == FAILURE) {
			RETURN_FALSE;
		}
		DATE_FROM_OBJECT(pdate, object);
		DATE_ASSERT_RETURN_OBJECT(date_add_months(pdate, (DateMonth) months), object);
	} else {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &date, &months) == FAILURE) {
			RETURN_FALSE;
		}

		ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);
		DATE_ASSERT_RETURN_BOOL( date_add_months(pdate, (DateMonth) months) );
	}
}
/* }}} */

/* {{{ proto bool date_sub_months(resource date, int months)
   sub months to 'date' */
PHP_FUNCTION(date_sub_months)
{
	long months;
	zval *date;
	Date *pdate;
	zval *object = getThis();

	if (object) {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &months) == FAILURE) {
			RETURN_FALSE;
		}
		DATE_FROM_OBJECT(pdate, object);
		DATE_ASSERT_RETURN_OBJECT( date_sub_months(pdate, (DateMonth) months), object);
	} else {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &date, &months) == FAILURE) {
			RETURN_FALSE;
		}

		ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);
		DATE_ASSERT_RETURN_BOOL( date_sub_months(pdate, (DateMonth) months) );
	}
}
/* }}} */

/* {{{ proto bool date_add_years(resource date, int years)
   add years to 'date' */
PHP_FUNCTION(date_add_years)
{
	long years;
	zval *date;
	Date *pdate;
	zval *object = getThis();

	if (object) {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &years) == FAILURE) {
			RETURN_FALSE;
		}
		DATE_FROM_OBJECT(pdate, object);
		DATE_ASSERT_RETURN_OBJECT( date_add_years(pdate, (DateYear) years), object);
	} else {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &date, &years) == FAILURE) {
			RETURN_FALSE;
		}

		ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);
		DATE_ASSERT_RETURN_BOOL( date_add_years(pdate, (DateYear) years) );
	}
}
/* }}} */

/* {{{ proto bool date_sub_years(resource date, int years)
   sub years to 'date' */
PHP_FUNCTION(date_sub_years)
{
	long years;
	zval *date;
	Date *pdate;
	zval *object = getThis();

	if (object) {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &years) == FAILURE) {
			RETURN_FALSE;
		}
		DATE_FROM_OBJECT(pdate, object);
		DATE_ASSERT_RETURN_OBJECT(date_sub_years(pdate, (DateYear) years), object);
	} else {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &date, &years) == FAILURE) {
			RETURN_FALSE;
		}

		ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);
		DATE_ASSERT_RETURN_BOOL(date_sub_years(pdate, (DateYear) years));
	}
}
/* }}} */

/* {{{ proto int date_days_between(resource date1, resource date2)
   rerturns the days between date1 and date2 */
PHP_FUNCTION(date_days_between)
{
	zval *date1, *date2;
	Date *pdate1,*pdate2;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rr", &date1, &date2) == FAILURE) {
		RETURN_FALSE;
	}

	ZEND_FETCH_RESOURCE(pdate1, Date *, &date1, -1, "date_time", le_date);
	ZEND_FETCH_RESOURCE(pdate2, Date *, &date2, -1, "date_time", le_date);

	RETURN_LONG((long)date_days_between(pdate1, pdate2));
}
/* }}} */

/* {{{ proto int date_compare(resource date1, resource date2)
   compare date1 to date2, 0 if equal, less than zero if date1 is less than date2, greater than zero if date1 is greater than date2 */
PHP_FUNCTION(date_compare)
{
	zval *date1, *date2;
	Date *pdate1, *pdate2;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rr", &date1, &date2) == FAILURE) {
		RETURN_FALSE;
	}

	ZEND_FETCH_RESOURCE(pdate1, Date *, &date1, -1, "date_time", le_date);
	ZEND_FETCH_RESOURCE(pdate2, Date *, &date2, -1, "date_time", le_date);

	RETURN_LONG((long)date_compare(pdate1, pdate2));
}
/* }}} */

/* {{{ proto void date_order(resource date1, resource date2)
   checks if date1 is less or equal to date2, swap them if not */
PHP_FUNCTION(date_order)
{
	zval *date1, *date2;
	Date *pdate1, *pdate2;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rr", &date1, &date2) == FAILURE) {
		RETURN_FALSE;
	}

	ZEND_FETCH_RESOURCE(pdate1, Date *, &date1, -1, "date_time", le_date);
	ZEND_FETCH_RESOURCE(pdate2, Date *, &date2, -1, "date_time", le_date);

	date_order(pdate1, pdate2);
}
/* }}} */

/* {{{ proto  date_clamp(resource s)
   date, resource date1, resource date2) clamps date between date1 and date2 */
PHP_FUNCTION(date_clamp)
{
	zval *date, *date1 = NULL, *date2 = NULL;
	Date *pdate, *pdate1 = NULL, *pdate2 = NULL;
	zval *object = getThis();

	if (object) {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z!|z!", &date1, &date2) == FAILURE) {
			RETURN_FALSE;
		}
		DATE_FROM_OBJECT(pdate, object);
	} else {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z|z!|z!", &date, &date1, &date2) == FAILURE) {
			RETURN_FALSE;
		}

		DATE_FROM_ZVAL(pdate, date);
	}

	if (date1) {
		DATE_FROM_ZVAL(pdate1, date1);
	}

	if (date2) {
		DATE_FROM_ZVAL(pdate2, date2);
	}

	date_clamp(pdate, pdate1, pdate2);
}
/* }}} */

/* {{{ proto int date_get_weekday(resource date)
   returns the day of the week of date 'date' */
PHP_FUNCTION(date_get_weekday)
{
	zval *date;
	Date *pdate;
	zval *object = getThis();

	if (object) {
		DATE_FROM_OBJECT(pdate, object);
	} else {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &date) == FAILURE) {
			RETURN_FALSE;
		}

		ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);
	}

	RETURN_LONG((long)date_get_weekday(pdate));
}
/* }}} */

/* {{{ proto int date_get_day_of_year(resource date)
   returns the day of the year of date 'date' */
PHP_FUNCTION(date_get_day_of_year)
{
	zval *date;
	Date *pdate;
	zval *object = getThis();

	if (object) {
		DATE_FROM_OBJECT(pdate, object);
	} else {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &date) == FAILURE) {
			RETURN_FALSE;
		}

		ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);
	}
	RETURN_LONG(date_get_day_of_year(pdate));
}
/* }}} */

/* {{{ proto int date_get_week_of_year(resource date)
   returns the week of the year of date 'date' */
PHP_FUNCTION(date_get_week_of_year)
{
	zval *date;
	Date *pdate;
	zval *object = getThis();

	if (object) {
		DATE_FROM_OBJECT(pdate, object);
	} else {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &date) == FAILURE) {
			RETURN_FALSE;
		}

		ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);
	}
	RETURN_LONG(date_get_week_of_year(pdate));
}
/* }}} */

/* {{{ proto int date_get_week_of_year(resource date)
   returns the week of the year of date 'date' */
PHP_FUNCTION(date_get_isoweek_of_year)
{
	zval *date;
	Date *pdate;
	zval *object = getThis();

	if (object) {
		DATE_FROM_OBJECT(pdate, object);
	} else {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &date) == FAILURE) {
			RETURN_FALSE;
		}

		ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);
	}
	RETURN_LONG(date_get_isoweek_of_year(pdate));
}

/* {{{ proto bool date_set_to_weekday(resource date, int weekday, int n [,int month[,int year]])
	Sets the date to the n-th weekday in the given month of the given year.
	The current month and year are used by default).
	'n' may be either positive (counting from the beginning of the month)
	or negative (counting from the end of it).
*/
PHP_FUNCTION(date_set_to_weekday)
{
	long weekday, n, month = 0, year = 0;
	zval *date;
	Date *pdate;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rll|ll", &date, &weekday, &n, &month, &year) == FAILURE) {
		RETURN_FALSE;
	}

	ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);

	if (month && !date_set_month(pdate, month)) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "invalid month");
		RETURN_FALSE;
	}

	if (year && !date_set_year(pdate, year)) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "invalid year");
		RETURN_FALSE;
	}

	DATE_ASSERT_RETURN_BOOL(date_set_to_weekday(pdate, (DateWeekday)weekday, (int)n));
}
/* }}} */

/* {{{ proto resource date_get_to_weekday(resource date, int weekday, int n [, int month [, int year]])
   see date_set_to_weekday */
PHP_FUNCTION(date_get_to_weekday)
{
	long weekday, n, month = 0, year = 0;
	zval *date;
	Date *pdate, *newdate = NULL;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rll|ll", &date, &weekday, &n, &month, &year) == FAILURE) {
		RETURN_FALSE;
	}

	ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);

	if (month && !date_set_month(pdate, month)){
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "invalid month");
		RETURN_FALSE;
	}
	if (year && !date_set_year(pdate, year)){
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "invalid year");
		RETURN_FALSE;
	}
	newdate = date_get_to_weekday(pdate, (DateWeekday)weekday, (int)n);
	DATE_REGISTER_RESOURCE(newdate);
}
/* }}} */

/* {{{ proto bool date_set_to_last_month_day(resource date)
   set 'date' to the last day of its months */
PHP_FUNCTION(date_set_to_last_month_day)
{
	zval *date;
	Date *pdate;
	zval *object = getThis();

	if (object) {
		DATE_FROM_OBJECT(pdate, object);
	} else {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &date) == FAILURE) {
			RETURN_FALSE;
		}

		ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);
	}
	RETURN_BOOL(date_set_to_lastmonthday(pdate));
}
/* }}} */

/* {{{ proto resource date_get_to_last_month_day(resource date)
   see set_to_lastmonthday */
PHP_FUNCTION(date_get_to_last_month_day)
{
	zval *date;
	Date *pdate, *newdate = NULL;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &date) == FAILURE) {
		RETURN_FALSE;
	}

	ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);

	newdate = date_get_to_lastmonthday(pdate);
	DATE_REGISTER_RESOURCE(newdate);
}
/* }}} */

/* {{{ proto bool date_set_to_week(ressource date, int week_number, int weekday)
*/
PHP_FUNCTION(date_set_to_week)
{
	zval *date;
	Date *pdate;
	long weekday, n;
	zval *object = getThis();

	if (object) {
		DATE_FROM_OBJECT(pdate, object);
	} else {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rll", &date,  &n, &weekday) == FAILURE) {
			RETURN_FALSE;
		}

		ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);
	}

	RETURN_BOOL(date_settotheweek(pdate, n, weekday));
}
/* }}} */

/* {{{ proto bool date_get_to_week(ressource date, int week_number, int weekday)
*/
PHP_FUNCTION(date_get_to_week)
{
	zval *date;
	Date *pdate, *newdate = NULL;
	long weekday, n;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rll", &date,  &n, &weekday) == FAILURE) {
		RETURN_FALSE;
	}

	ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);

	newdate = date_gettotheweek(pdate);
	DATE_REGISTER_RESOURCE(newdate);
}
/* }}} */

/* {{{ proto bool date_set_to_prev_week_day(resource date, int weekday)
   set 'date' to the previous same weekday */
PHP_FUNCTION(date_set_to_prev_week_day)
{
	long weekday;
	zval *date;
	Date *pdate;
	zval *object = getThis();

	if (object) {
		DATE_FROM_OBJECT(pdate, object);
	} else {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &date, &weekday) == FAILURE) {
			RETURN_FALSE;
		}

		ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);
	}
	RETURN_BOOL(date_settoprevweekday(pdate, (DateWeekday)weekday));
}
/* }}} */

/* {{{ proto resource date_get_to_prev_week_day(resource date, int weekday)
   see settoprevweekday */
PHP_FUNCTION(date_get_to_prev_week_day)
{
	long weekday;
	zval *date;
	Date *pdate, *newdate = NULL;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &date, &weekday) == FAILURE) {
		RETURN_FALSE;
	}

	ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);

	newdate = date_gettoprevweekday(pdate, weekday);
	DATE_REGISTER_RESOURCE(newdate);
}
/* }}} */

/* {{{ proto bool date_set_to_next_weekday(resource date, int weekday)
   set 'date' to the next same weekday */
PHP_FUNCTION(date_set_to_next_weekday)
{
	long weekday;
	zval *date;
	Date *pdate;
	zval *object = getThis();

	if (object) {
		DATE_FROM_OBJECT(pdate, object);
	} else {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &date, &weekday) == FAILURE) {
			RETURN_FALSE;
		}

		ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);
	}
	RETURN_BOOL(date_settonextweekday(pdate, (DateWeekday)weekday));
}
/* }}} */

/* {{{ proto resource date_get_to_next_weekday(resource date, int weekday)
   see settonextweekday */
PHP_FUNCTION(date_get_to_next_weekday)
{
	long weekday;
	zval *date;
	Date *pdate, *newdate = NULL;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &date, &weekday) == FAILURE) {
		RETURN_FALSE;
	}

	ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);

	newdate = date_gettonextweekday(pdate, weekday);
	DATE_REGISTER_RESOURCE(newdate);
}
/* }}} */

/* {{{ proto bool date_set_to_weekday_in_same_week(resource date, int weekday)
   set 'date' to the weekday 'wd' in the same week */
PHP_FUNCTION(date_set_to_weekday_in_same_week)
{
	long weekday;
	zval *date;
	Date *pdate;
	zval *object = getThis();

	if (object) {
		DATE_FROM_OBJECT(pdate, object);
	} else {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &date, &weekday) == FAILURE) {
			RETURN_FALSE;
		}

		ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);
	}
	RETURN_BOOL(date_settoweekdayinsameweek(pdate, (DateWeekday) weekday));
}
/* }}} */

/* {{{ proto resource date_get_to_weekday_in_same_week(resource date, int weekday)
   see settoweekdayinsameweek */
PHP_FUNCTION(date_get_to_weekday_in_same_week)
{
	long weekday;
	zval *date;
	Date *pdate, *newdate = NULL;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &date, &weekday) == FAILURE) {
		RETURN_FALSE;
	}

	ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);

	newdate = date_gettoweekdayinsameweek(pdate, (DateWeekday)weekday);
	DATE_REGISTER_RESOURCE(newdate);
}
/* }}} */

/* {{{ proto bool date_set_last_dow(resource date)
   set 'date' to the last day of its week */
PHP_FUNCTION(date_set_last_dow)
{
	zval *date;
	Date *pdate;
	zval *object = getThis();

	if (object) {
		DATE_FROM_OBJECT(pdate, object);
	} else {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &date) == FAILURE) {
			RETURN_FALSE;
		}
		ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);
	}
	RETURN_BOOL(date_set_last_dow(pdate));
}
/* }}} */

/* {{{ proto resource date_get_last_dow(resource date)
   see set_last_dow */
PHP_FUNCTION(date_get_last_dow)
{
	zval *date;
	Date *pdate, *newdate = NULL;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &date) == FAILURE) {
		RETURN_FALSE;
	}

	ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);

	newdate = date_get_last_dow(pdate);
	DATE_REGISTER_RESOURCE(newdate);
}
/* }}} */

/* {{{ proto bool date_set_first_dow(resource date)
   set 'date' to the first day of its week */
PHP_FUNCTION(date_set_first_dow)
{
	zval *date;
	Date *pdate;
	zval *object = getThis();

	if (object) {
		DATE_FROM_OBJECT(pdate, object);
	} else {
		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &date) == FAILURE) {
			RETURN_FALSE;
		}

		ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);
	}

	RETURN_BOOL(date_set_first_dow(pdate));
}
/* }}} */

/* {{{ proto resource date_get_first_dow(resource date)
   see set_first_dow */
PHP_FUNCTION(date_get_first_dow)
{
	zval *date;
	Date *pdate, *newdate = NULL;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &date) == FAILURE) {
		RETURN_FALSE;
	}

	ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);

	newdate = date_get_first_dow(pdate);
	DATE_REGISTER_RESOURCE(newdate);
}
/* }}} */

/* {{{ proto int date_get_days_in_month(int month, int year)
   returns the number of days in the month 'm' of year 'y' */
PHP_FUNCTION(date_get_days_in_month)
{
	long month, year;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ll", &month, &year) == FAILURE) {
		RETURN_FALSE;
	}

	RETURN_LONG(date_get_days_in_month((DateMonth)month,(DateYear)year));
}
/* }}} */

/* {{{ proto int date_get_weeks_of_year(int year)
   returns the number of weeks in year 'year' */
PHP_FUNCTION(date_get_weeks_of_year)
{
	long year;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &year) == FAILURE) {
		RETURN_FALSE;
	}

	RETURN_LONG(date_get_weeks_of_year((DateYear)year));
}
/* }}} */

/* {{{ proto string date_strftime()
   ressource date, string format) see strftime */
PHP_FUNCTION(date_strftime)
{
	if (ZEND_NUM_ARGS() != 0) {
		WRONG_PARAM_COUNT;
	}
	php_error(E_WARNING, "date_strftime: not yet implemented");
}
/* }}} */

/* {{{ proto string date_strftime()
   ressource date, string format) see strftime */
PHP_FUNCTION(date_get_array)
{
	zval *date;
	Date *pdate;
	zval *object = getThis();

	if (object) {
		DATE_FROM_OBJECT(pdate, object);
	} else if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &date) != FAILURE) {
		ZEND_FETCH_RESOURCE(pdate, Date *, &date, -1, "date_time", le_date);
	} else {
		RETURN_FALSE;
	}

	array_init(return_value);

	add_assoc_long(return_value, "day",	date_get_day(pdate));
	add_assoc_long(return_value, "month",	date_get_month(pdate));
	add_assoc_long(return_value, "year",	date_get_year(pdate));
	add_assoc_long(return_value, "hour",	0);
	add_assoc_long(return_value, "min",	0);
	add_assoc_long(return_value, "sec",	0);
}
/* }}} */

/* {{{ proto string date_get_easter(int year)
   returns the easter date of the given year */
PHP_FUNCTION(date_get_easter)
{
	Date *easter;
	long year, weekstart = DATE_STARTWEEK_MONDAY;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l|l", &year, &weekstart) == FAILURE) {
		RETURN_FALSE;
	}
	easter = date_get_easter(year, weekstart);
	DATE_REGISTER_RESOURCE(easter);
}
/* }}} */

/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
