/*
  +----------------------------------------------------------------------+
  | PHP Version 5                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2004 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3.0 of the PHP license,       |
  | that is bundled with this package in the file LICENSE, and is        |
  | available at through the world-wide-web at                           |
  | http://www.php.net/license/3_0.txt.                                  |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Author: Pierre-Alain Joye <paj@pearfr.org>                           |
  +----------------------------------------------------------------------+

  $Id: t_date.h,v 1.5 2004/03/05 12:21:10 pajoye Exp $
*/
#ifndef T_DATE_H
#define T_DATE_H

/* Returns val if expr fails */
#define date_return_val_if_failed(expr, val) \
	if (!(expr)) return val;

/* Returns if expr fails */
# define date_return_if_failed(expr) \
	if (!(expr)) return;

typedef struct _date Date;
typedef unsigned int date_uint;
typedef int	date_int;
typedef date_uint DateBoolean;

typedef date_uint DateMonth;
typedef date_int DateYear;
typedef date_uint DateDay;   /* day of the month */
typedef date_uint DateWeek;
#define DATE_TRUE	1
#define DATE_FALSE	0

#define DATE_STARTWEEK_SUNDAY 0
#define DATE_STARTWEEK_MONDAY 1

#define DATE_JULIAN_MIN  1
#define DATE_JULIAN_MAX  23936166

#define DATE_YEAR_MIN 1
#define DATE_YEAR_MAX 65535

#define DATE_MONTH_MIN 1
#define DATE_MONTH_MAX 31

#define DATE_BAD_JULIAN 0U
#define DATE_BAD_DAY    0U
#define DATE_BAD_YEAR   0U

struct _date{
	/* julian days representation - we use a
	 *  bitfield hoping that 64 bit platforms
	 *  will pack this whole struct in one big
	 *  int
	 */
	date_uint julian_days : 32;

	/* julian is valid */
	date_uint julian	: 1;

	/* dmy is valid */
	date_uint dmy		: 1;

	/* DMY representation */
	date_uint day		: 6;
	date_uint month		: 4;
	date_uint year		: 16;

	/* Time data, not yet implemented */
	int hours;
	int minutes;
	int seconds;
	int mseconds;

	/* 1: start on monday, 0 on sunday */
	date_uint weekstart	: 1;

	/* status
	 * 1  date is valid (or not uptodate)
	 * -1 invalid date
	 */
	int status;
};

typedef enum {
	DATE_BAD_WEEKDAY  = 0,
	DATE_MONDAY       = 1,
	DATE_TUESDAY      = 2,
	DATE_WEDNESDAY    = 3,
	DATE_THURSDAY     = 4,
	DATE_FRIDAY       = 5,
	DATE_SATURDAY     = 6,
	DATE_SUNDAY       = 7
}  DateWeekday;

typedef enum {
	DATE_BAD_MONTH = 0,
	DATE_JANUARY   = 1,
	DATE_FEBRUARY  = 2,
	DATE_MARCH     = 3,
	DATE_APRIL     = 4,
	DATE_MAY       = 5,
	DATE_JUNE      = 6,
	DATE_JULY      = 7,
	DATE_AUGUST    = 8,
	DATE_SEPTEMBER = 9,
	DATE_OCTOBER   = 10,
	DATE_NOVEMBER  = 11,
	DATE_DECEMBER  = 12
} Date_Month;

static const date_uint days_in_months[2][13] = {
	/* error, jan feb mar apr may jun jul aug sep oct nov dec */
	{  0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 },
	{  0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 } /* leap year */
};

static const date_uint days_in_year[2][14] = {
	/* 0, jan feb mar apr may  jun  jul  aug  sep  oct  nov  dec */
	{  0, 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365 },
	{  0, 0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366 } /* leap year */
};
/*
date_uint date_weekstart;
*/
/* dates create, init, destroy */
Date * date_new();
void date_free( Date *d);
void date_clear(Date *d, int count);
Date *date_new_julian(date_uint julian, DateDay weekstart TSRMLS_DC);
Date *date_new_dmy(DateDay d, DateMonth m, DateYear y, DateDay weekstart);
Date *date_new_now(DateDay weekstart TSRMLS_DC);

/* misc validations functions */
DateBoolean date_valid (const Date *d);
DateBoolean date_valid_julian (int j);
DateBoolean date_valid_dmy (DateDay d, DateMonth m, DateYear y);

/* dates data checks */
DateBoolean date_valid_day(DateDay d);
DateBoolean date_valid_month(DateMonth m);
DateBoolean date_valid_year(DateYear y);
DateBoolean date_valid_weekday(DateWeekday wd);

/* set/get Date properties */
DateBoolean date_set_dmy(Date *d, DateDay day, DateMonth m, DateYear y);
DateBoolean date_set_day(Date *d, DateDay day);
DateDay date_get_day(Date *d);

DateBoolean date_set_month(Date *d, DateMonth month);
DateMonth date_get_month(Date *d);

DateBoolean date_set_year(Date *d, DateYear year);
DateYear date_get_year(Date *d);

DateBoolean date_set_julian(Date *d, int j);

int date_get_julian(Date *d);

/* int date_get_century(Date *d); */

/* update datas */
void date_update_dmy(const Date *const_d);
void date_update_julian(const Date *const_d);

/* date properties check */
DateBoolean date_is_leap_year(DateYear year);

DateBoolean date_is_first_of_month(Date *d);
DateBoolean date_is_last_of_month(Date *d);

/* +/- operators */
DateBoolean date_add_days(Date *d, int days);
DateBoolean date_sub_days(Date *d, int days);


DateBoolean date_add_weeks(Date *d, int nweeks);
DateBoolean date_sub_weeks(Date *d, int nweeks);

DateBoolean date_add_months(Date *d, int nmonths);
DateBoolean date_sub_months(Date *d, int nmonths);

DateBoolean date_add_years(Date *d, int nyears);
DateBoolean date_sub_years(Date *d, int nyears);

/* calculations */
DateDay date_days_between (Date *d1,Date *d2);

/* sort, swap, compare functions */
int date_compare(Date *d1, Date *d2);
void date_order (Date *d1,Date *d2);
void date_clamp(Date *date, Date *min_date, Date *max_date);


/* various set/get */
DateWeekday date_get_weekday(Date *d);
int date_get_day_of_year(Date *d);
int date_get_week_of_year(Date *d);
int date_get_isoweek_of_year(Date *date);

DateBoolean date_set_to_weekday(Date *d, DateWeekday weekday, int n);
Date *date_get_to_weekday(Date *d, DateWeekday weekday, int n);

DateBoolean date_settolastweekday(Date *d);
Date* date_gettolastweekday(Date *d);

DateBoolean date_set_to_lastmonthday(Date *d);
Date* date_get_to_lastmonthday(Date *d);

DateBoolean date_settotheweek();
Date* date_gettotheweek();

DateBoolean date_settoprevweekday(Date *d, DateWeekday wd);
Date* date_gettoprevweekday(Date *d, DateWeekday wd);

DateBoolean date_settonextweekday(Date *d, DateWeekday wd);
Date* date_gettonextweekday(Date *d, DateWeekday wd);

DateBoolean date_settoweekdayinsameweek(Date *d, DateWeekday wd);
Date* date_gettoweekdayinsameweek(Date *d, DateWeekday wd);

DateBoolean date_set_last_dow(Date *d);
Date* date_get_last_dow(Date *d);

DateBoolean date_set_first_dow(Date *d);
Date* date_get_first_dow(Date *d);

/*DateBoolean date_set_to_weekday(Date *d, DateWeekday weekday, int n);
 Date *date_get_to_weekday(Date *d, DateWeekday weekday, int n); */

/*
DateBoolean date_set_to_lastday(Date *d);
Date* date_get_to_lastday(Date *d);
*/
/* misc date related functions */
int date_get_days_in_month(DateMonth month, DateYear year);
int date_get_weeks_of_year (DateYear year);

Date *date_get_easter(DateYear year, DateWeekday weekstart);

#endif
